import { DataSource, MigrationInterface, QueryRunner, View } from "typeorm";
import { member } from "../entity/club/member/member";
import { memberExecutivePositions } from "../entity/club/member/memberExecutivePositions";
import { memberQualifications } from "../entity/club/member/memberQualifications";
import { membership } from "../entity/club/member/membership";

export class UpdateViews1737800468938 implements MigrationInterface {
  name = "UpdateViews1737800468938";

  public async up(queryRunner: QueryRunner): Promise<void> {
    // await queryRunner.dropView("membership_view");
    // await queryRunner.dropView("member_executive_positions_view");
    // await queryRunner.dropView("member_qualifications_view");
    // await queryRunner.dropView("member_view");
    await queryRunner.query(`DROP VIEW IF EXISTS membership_view`);
    await queryRunner.query(`DROP VIEW IF EXISTS member_executive_positions_view`);
    await queryRunner.query(`DROP VIEW IF EXISTS member_qualifications_view`);
    await queryRunner.query(`DROP VIEW IF EXISTS member_view`);

    await queryRunner.createView(
      new View({
        name: "member_view",
        expression: (datasource: DataSource) =>
          datasource
            .getRepository(member)
            .createQueryBuilder("member")
            .select("member.id", "id")
            .addSelect("member.firstname", "firstname")
            .addSelect("member.lastname", "lastname")
            .addSelect("member.nameaffix", "nameaffix")
            .addSelect("member.birthdate", "birthdate")
            .addSelect("salutation.salutation", "salutation")
            .addSelect("TIMESTAMPDIFF(YEAR, member.birthdate, CURDATE())", "todayAge")
            .addSelect("YEAR(CURDATE()) - YEAR(member.birthdate)", "ageThisYear")
            .addSelect("CONCAT('_', FROM_DAYS(TIMESTAMPDIFF(DAY, member.birthdate, CURDATE())))", "exactAge")
            .leftJoin("member.salutation", "salutation"),
      }),
      true
    );
    await queryRunner.createView(
      new View({
        name: "member_executive_positions_view",
        expression: (datasource: DataSource) =>
          datasource
            .getRepository(memberExecutivePositions)
            .createQueryBuilder("memberExecutivePositions")
            .select("executivePosition.id", "positionId")
            .addSelect("executivePosition.position", "position")
            .addSelect("member.id", "memberId")
            .addSelect("member.firstname", "memberFirstname")
            .addSelect("member.lastname", "memberLastname")
            .addSelect("member.nameaffix", "memberNameaffix")
            .addSelect("member.birthdate", "memberBirthdate")
            .addSelect("salutation.salutation", "memberSalutation")
            .addSelect(
              "SUM(TIMESTAMPDIFF(DAY, memberExecutivePositions.start, COALESCE(memberExecutivePositions.end, CURRENT_DATE)))",
              "durationInDays"
            )
            .leftJoin("memberExecutivePositions.executivePosition", "executivePosition")
            .leftJoin("memberExecutivePositions.member", "member")
            .leftJoin("member.salutation", "salutation")
            .groupBy("executivePosition.id")
            .addGroupBy("member.id"),
      }),
      true
    );
    await queryRunner.createView(
      new View({
        name: "member_qualifications_view",
        expression: (datasource: DataSource) =>
          datasource
            .getRepository(memberQualifications)
            .createQueryBuilder("memberQualifications")
            .select("qualification.id", "qualificationId")
            .addSelect("qualification.qualification", "qualification")
            .addSelect("member.id", "memberId")
            .addSelect("member.firstname", "memberFirstname")
            .addSelect("member.lastname", "memberLastname")
            .addSelect("member.nameaffix", "memberNameaffix")
            .addSelect("member.birthdate", "memberBirthdate")
            .addSelect("salutation.salutation", "memberSalutation")
            .addSelect(
              "SUM(TIMESTAMPDIFF(DAY, memberQualifications.start, COALESCE(memberQualifications.end, CURRENT_DATE)))",
              "durationInDays"
            )
            .leftJoin("memberQualifications.qualification", "qualification")
            .leftJoin("memberQualifications.member", "member")
            .leftJoin("member.salutation", "salutation")
            .groupBy("qualification.id")
            .addGroupBy("member.id"),
      }),
      true
    );
    await queryRunner.createView(
      new View({
        name: "membership_view",
        expression: (datasource: DataSource) =>
          datasource
            .getRepository(membership)
            .createQueryBuilder("membership")
            .select("status.id", "statusId")
            .addSelect("status.status", "status")
            .addSelect("member.id", "memberId")
            .addSelect("member.firstname", "memberFirstname")
            .addSelect("member.lastname", "memberLastname")
            .addSelect("member.nameaffix", "memberNameaffix")
            .addSelect("member.birthdate", "memberBirthdate")
            .addSelect("salutation.salutation", "memberSalutation")
            .addSelect(
              "SUM(TIMESTAMPDIFF(DAY, membership.start, COALESCE(membership.end, CURRENT_DATE)))",
              "durationInDays"
            )
            .addSelect(
              "CONCAT('_', FROM_DAYS(SUM(TIMESTAMPDIFF(DAY, membership.start, COALESCE(membership.end, CURRENT_DATE)))))",
              "durationInYears"
            )
            .leftJoin("membership.status", "status")
            .leftJoin("membership.member", "member")
            .leftJoin("member.salutation", "salutation")
            .groupBy("status.id")
            .addGroupBy("member.id"),
      }),
      true
    );
  }

  public async down(queryRunner: QueryRunner): Promise<void> {
    await queryRunner.dropView("membership_view");
    await queryRunner.dropView("member_executive_positions_view");
    await queryRunner.dropView("member_qualifications_view");
    await queryRunner.dropView("member_view");

    await queryRunner.createView(
      new View({
        name: "member_view",
        expression: (datasource: DataSource) =>
          datasource
            .getRepository(member)
            .createQueryBuilder("member")
            .select("member.id", "id")
            .addSelect("member.salutation", "salutation")
            .addSelect("member.firstname", "firstname")
            .addSelect("member.lastname", "lastname")
            .addSelect("member.nameaffix", "nameaffix")
            .addSelect("member.birthdate", "birthdate")
            .addSelect("TIMESTAMPDIFF(YEAR, member.birthdate, CURDATE())", "todayAge")
            .addSelect("YEAR(CURDATE()) - YEAR(member.birthdate)", "ageThisYear")
            .addSelect("CONCAT('_', FROM_DAYS(TIMESTAMPDIFF(DAY, member.birthdate, CURDATE())))", "exactAge"),
      }),
      true
    );
    await queryRunner.createView(
      new View({
        name: "member_executive_positions_view",
        expression: (datasource: DataSource) =>
          datasource
            .getRepository(memberExecutivePositions)
            .createQueryBuilder("memberExecutivePositions")
            .select("executivePosition.id", "positionId")
            .addSelect("executivePosition.position", "position")
            .addSelect("member.id", "memberId")
            .addSelect("member.salutation", "memberSalutation")
            .addSelect("member.firstname", "memberFirstname")
            .addSelect("member.lastname", "memberLastname")
            .addSelect("member.nameaffix", "memberNameaffix")
            .addSelect("member.birthdate", "memberBirthdate")
            .addSelect(
              "SUM(TIMESTAMPDIFF(DAY, memberExecutivePositions.start, COALESCE(memberExecutivePositions.end, CURRENT_DATE)))",
              "durationInDays"
            )
            .leftJoin("memberExecutivePositions.executivePosition", "executivePosition")
            .leftJoin("memberExecutivePositions.member", "member")
            .groupBy("executivePosition.id"),
      }),
      true
    );
    await queryRunner.createView(
      new View({
        name: "member_qualifications_view",
        expression: (datasource: DataSource) =>
          datasource
            .getRepository(memberQualifications)
            .createQueryBuilder("memberQualifications")
            .select("qualification.id", "qualificationId")
            .addSelect("qualification.qualification", "qualification")
            .addSelect("member.id", "memberId")
            .addSelect("member.salutation", "memberSalutation")
            .addSelect("member.firstname", "memberFirstname")
            .addSelect("member.lastname", "memberLastname")
            .addSelect("member.nameaffix", "memberNameaffix")
            .addSelect("member.birthdate", "memberBirthdate")
            .addSelect(
              "SUM(TIMESTAMPDIFF(DAY, memberQualifications.start, COALESCE(memberQualifications.end, CURRENT_DATE)))",
              "durationInDays"
            )
            .leftJoin("memberQualifications.qualification", "qualification")
            .leftJoin("memberQualifications.member", "member")
            .groupBy("qualification.id"),
      }),
      true
    );
    await queryRunner.createView(
      new View({
        name: "membership_view",
        expression: (datasource: DataSource) =>
          datasource
            .getRepository(membership)
            .createQueryBuilder("membership")
            .select("status.id", "statusId")
            .addSelect("status.status", "status")
            .addSelect("member.id", "memberId")
            .addSelect("member.salutation", "memberSalutation")
            .addSelect("member.firstname", "memberFirstname")
            .addSelect("member.lastname", "memberLastname")
            .addSelect("member.nameaffix", "memberNameaffix")
            .addSelect("member.birthdate", "memberBirthdate")
            .addSelect(
              "SUM(TIMESTAMPDIFF(DAY, membership.start, COALESCE(membership.end, CURRENT_DATE)))",
              "durationInDays"
            )
            .addSelect(
              "CONCAT('_', FROM_DAYS(SUM(TIMESTAMPDIFF(DAY, membership.start, COALESCE(membership.end, CURRENT_DATE)))))",
              "durationInYears"
            )
            .leftJoin("membership.status", "status")
            .leftJoin("membership.member", "member")
            .groupBy("status.id")
            .addGroupBy("member.id"),
      }),
      true
    );
  }
}
